"""Top-level package for digital-diffeomorphism."""
from digital_diffeomorphism.core import ndv, nda

__title__ = "digital-diffeomorphism"
__description__ = "Digital diffeomorphism: Non-diffeomorphic volume and non-diffeomorphic area computation"
__url__ = "https://github.com/yihao6/digital_diffeomorphism"
__author__ = """Yihao Liu"""
__email__ = "yliu236@jhu.edu"
__version__ = "1.1.1"
__license__ = "GPLv3"
__copyright__ = "Copyright 2023 Yihao Liu"
